function [smpval,smp,gcv,gcvtop,gcvbot] = gcv_plot_box(a,b,we,wm_or_c,is_wm)

% @(#)   gcv_plot_box.m  version 1.5  created 92/12/15
%-----------------------------------------------------------------------------
% FUNCTION [smpval,smp,gcv,gcvtop,gcvbot] = gcv_plot_box(a,b,we,wm_or_c,is_wm)
%
% Plots GCV as a function of log10(smp)
%
% Input:  a         matrix					(MxN)
%         b         right-hand-side vector			(Mx1)
%         we        square of row (equation) weight matrix	(MxM)
%         wm_or_c   norm/semi-norm or column weight matrix	(NxN)
%         is_wm     1 if wm_or_c is norm/semi-norm weight matrix
%                   otherwise, assumed to be column weight matrix
%
% Output: smpval    chosen value of smoothing parameter
%         smp       vector of smoothing parameters
%         gcv       vector of gcv values
%         gcvtop    vector of gcv numerators   (e'*We*e/m)
%         gcvbot    vector of gcv denominators ({1-trace(R)/m}^2)
%
% NOTE:   smp defined as in A'*A + smp^2*I
%
% Calls:            gcv_func1_box, gcv_func2_box, gcv_method_box,
%                   save_graph_box, seeplot_box
% Dependency:       Must have called gcv_global_box to set global variables
%-----------------------------------------------------------------------------

% Reference: Wahba & Wendelberger (1980), McIntosh & Veronis (1993)
% Author:    Peter McIntosh
% History:   See SCCS comments

%-----------------------------------------------------------------------------

%...Check that gcv_global_box has been called previously...

test=exist('glb_a');
if test~=1
  error('Global variables not defined - call gcv_global_box first');
end

%...Decide on method to use...

glb_gcv_method = gcv_method_box(we,wm_or_c,is_wm);

%...Do some preliminary computation...

if glb_gcv_method==1
  if is_wm~=1,
    cinv=inv(wm_or_c);
    glb_wm=cinv*cinv;
  else
    glb_wm=wm_or_c;
  end
  at_we=a'*we;
  glb_at_we_a=at_we*a;
  glb_at_we_b=at_we*b;
elseif glb_gcv_method==2
  if is_wm~=1,
    glb_wm_inv_at=wm_or_c*wm_or_c*a';
  else
    glb_wm_inv_at=wm_or_c\a';
  end
  glb_we_inv=inv(we);
  glb_a_wm_inv_at=a*glb_wm_inv_at;
end

[m,n]=size(a);

glb_a=a;
glb_b=b;
glb_we=we;

nr=20;
nrin=input('Enter number of points [CR=20] > ');
if ~isempty(nrin),nr=nrin;end

gcv        = zeros(nr,1);
gcvtop     = zeros(nr,1);
gcvbot     = zeros(nr,1);
trr        = zeros(nr,1);
mu1        = zeros(nr,1);
mu1sqonmu2 = zeros(nr,1);
h          = zeros(nr,1);

gcvloop=1;
while gcvloop==1

  lmin=input('GCV PLOT: enter lower limit for log10(smp) > ');
  lmax=input('GCV PLOT: enter upper limit for log10(smp) > ');
  smp=logspace(lmin,lmax,nr)';
  
  for i=1:nr
    logsmp=log10(smp(i));
    if glb_gcv_method==1,
      [gcv(i),gcvtop(i),gcvbot(i),trr(i),e,r] = gcv_func1_box(logsmp);
    elseif glb_gcv_method==2,
      [gcv(i),gcvtop(i),gcvbot(i),trr(i),e,r] = gcv_func2_box(logsmp);
    end

    mu1(i)        = trace(r)/m;
    mu2           = trace(r*r)/m;
    mu1sqonmu2(i) = mu1(i)*mu1(i)/mu2;
    h(i)          = (2*mu1(i)+mu1sqonmu2(i))/((1-mu1(i))^2);

  end
  
  clg;

%...Graphs ordered so that ginput works off scale of last graph...

  subplot(224);
  semilogx(smp,mu1sqonmu2);
  grid;
  xlabel('smp');
  ylabel('mu1^2/mu2');
  title('mu1^2/mu2');
  
  subplot(223);
  semilogx(smp,mu1);
  grid;
  xlabel('smp');
  ylabel('mu1');
  title('mu1=trace(r)/m');
  
  subplot(222);
  loglog(smp,h);
  grid;
  xlabel('smp');
  ylabel('h');
  title('h');
  
  subplot(221);
  semilogx(smp,gcv);
  grid;
  xlabel('smp');
  ylabel('GCV');
  title('GCV plot');

  text(0.45,0.01,['method ',int2str(glb_gcv_method)],'sc')

  hold off;
  seeplot_box;

  reply=menu('GCV PLOT (CR to exit)',...
             'Proceed to choosing smp using ginput',...
             'Re-enter smp range');

  if reply==1
    break;
  elseif reply==2
    % Will continue in loop to re-enter smp range
  elseif isempty(reply)
    save_graph_box;
    subplot(111);
    return;
  else
    % Will also continue in loop to re-enter smp range
  end

end

limits=axis;axis;

inputloop=1;
while inputloop==1
  checkloop=1;
  while checkloop==1
    disp(' ')
    disp('Use crosshairs to choose smoothing parameter from 1st graph only')
    smpval=ginput(1);
%   ...Test that smp was chosen from 1st graph only...
    if smpval <= 10^limits(2) & smpval >= 10^limits(1)
      break;
    end
  end

  disp(' ')
  disp(['Smoothing parameter chosen = ',num2str(smpval)])
  reply=input('Use this smoothing parameter ? (y/n) > ','s');
  if strcmp(reply,'y')
    break;
  end
end

save_graph_box;
subplot(111);
%-----------------------------------------------------------------------------
