function [x,smpmin,gcvmin,r_model,r_data,s] = ...
                               gcv_soln_box(a,b,we,wm_or_c,is_wm,diagnose)

% @(#)   gcv_soln_box.m  version 1.2  created 92/10/20
%-----------------------------------------------------------------------------
% FUNCTION [x,smpmin,gcvmin,r_model,r_data,s] = ...
%                                gcv_soln_box(a,b,we,wm_or_c,is_wm,diagnose)
%
% Finds optimum ridge analysis solution to Ax=b  by minimizing GCV
% Solution is x = [A'*We*A + smpmin^2 * Wm]^-1 * A'*We*b
% where smpmin is the smoothing parameter which minimizes GCV.
%
% Input:  a         matrix					(MxN)
%         b         right-hand-side vector			(Mx1)
%         we        square of row (equation) weight matrix	(MxM)
%         wm_or_c   norm/semi-norm or column weight matrix	(NxN)
%         is_wm     1 if wm_or_c is norm/semi-norm weight matrix
%                   otherwise, assumed to be column weight matrix
%         diagnose  =1 to calculate r_model, r_data, s
%                   ~=1 for speed
%
% Output: x         solution vector				(Nx1)
%         smpmin    smoothing parameter which minimizes GCV	(scalar)
%         gcvmin    minimum value of GCV			(scalar)
%         r_model   model resolution matrix			(NxN)
%         r_data    data resolution matrix			(MxM)
%         s         degree of smoothing of data (%)		(scalar)
%
% Calls:            gcv_func1_box, gcv_func2_box, gcv_method_box,
%                   ridge_soln_box
% Dependency:       Must have called gcv_global_box to set global variables
%-----------------------------------------------------------------------------

% Reference: Wahba & Wendelberger (1980), McIntosh & Veronis (1993)
% Author:    Peter McIntosh
% History:   See SCCS comments

%-----------------------------------------------------------------------------

%...Check that gcv_global_box has been called previously...

test=exist('glb_a');
if test~=1
  error('Global variables not defined - call gcv_global_box first');
end

if nargin==5,
  diagnose=0;
end

%...Decide on method to use...

glb_gcv_method = gcv_method_box(we,wm_or_c,is_wm);

%...Do some preliminary computation...

if glb_gcv_method==1
  if is_wm~=1,
    cinv=inv(wm_or_c);
    glb_wm=cinv*cinv;
  else
    glb_wm=wm_or_c;
  end
  at_we=a'*we;
  glb_at_we_a=at_we*a;
  glb_at_we_b=at_we*b;
elseif glb_gcv_method==2
  if is_wm~=1,
    glb_wm_inv_at=wm_or_c*wm_or_c*a';
  else
    glb_wm_inv_at=wm_or_c\a';
  end
  glb_we_inv=inv(we);
  glb_a_wm_inv_at=a*glb_wm_inv_at;
end

glb_a=a;
glb_b=b;
glb_we=we;

lmin=input('GCV SOLN: enter lower search limit for log10(smp) > ');
lmax=input('GCV SOLN: enter upper search limit for log10(smp) > ');

%...Set "display" to be non-zero to display algorithm steps...

reply=input('Show minimization algorithm steps ? (y/n) [n] > ','s');

if strcmp(reply,'y'),
  display=1;
else
  display=0;
end


if glb_gcv_method==1,
  logsmp=fmin('gcv_func1_box',lmin,lmax,display);
  smpmin=10^logsmp;
  gcvmin=gcv_func1_box(logsmp);
elseif glb_gcv_method==2,
  logsmp=fmin('gcv_func2_box',lmin,lmax,display);
  smpmin=10^logsmp;
  gcvmin=gcv_func2_box(logsmp);
end

%...Store solution to test against ridge solution...

xgcv=glb_x;

%...Obtain final answer and diagnostics using ridge_soln_box...

[x,r_model,r_data,s] = ridge_soln_box(a,b,we,wm_or_c,is_wm,smpmin);

%...Check solution...

test=norm(x-xgcv)/sqrt(norm(x)*norm(xgcv));
tol=1.e-5;
if test>tol
  error('Conflict between ridge and gcv solutions');
end
%-----------------------------------------------------------------------------

