function wm = make_wm_box(dim,m,lscale)

% @(#)   make_wm_box.m  version 1.1  created 92/12/16
%----------------------------------------------------------------------------
% FUNCTION  wm = make_wm_box(dim,m,lscale)
%
% Constructs norm or semi-norm derivative weight matrix of order m.
%
% Input:  dim     number of spatially contiguous elements in vector to be
%                 smoothed e.g. if vector has 35 followed by 42 spatially
%                 contiguous elements, then dim = [35 42].
%         m       differential order of norm. This can either be a scalar,
%                 or a vector of the same length as dim.
%                 (+)ve for norm, (-)ve for semi-norm
%                 (MUST be scalar for norm at the moment).
%         lscale  length-scale (in grid units). Assumes uniform grid.
%                 Not needed for semi-norm.
%                 (MUST be scalar at the moment).
%
% Output: wm      derivative weight matrix
%                 norm:       wm = D0'*D0 + D1'*D1 + ... + Dm'*Dm
%                 semi-norm:  wm = Dm'*Dm
%                 where Di is i'th difference operator.
%
% Calls: make_sn
%----------------------------------------------------------------------------

% Reference: McIntosh and Veronis (1993)
% Author:    Peter McIntosh
% History:   PCM 17/09/91 - created
%            PCM 14/07/92 - order can now be a vector (semi-norm only!)
%            PCM 19/10/92 - test for lscale - must be a scalar at the moment.

%----------------------------------------------------------------------------

order=abs(m);

if length(find(m>0))==0
  wm = make_sn(dim,order);
else
  if nargin<3,
    error('make_wm_box: not enough input arguments.');
  end
  if length(order)>1,
    error('make_wm_box: norm vector differential order not implemented yet!');
  end
  if length(lscale)>1,
    error('make_wm_box: vector length scale not implemented yet!');
  end

  %...Define binomial coefficients for norm...
  
  for i=0:order
    coeff(i+1)=gamma(order+1)/(gamma(order-i+1)*gamma(i+1));
  end
  
  %...Construct norm weight matrix...
  
  wm=zeros(sum(dim));
  for i=0:order
    add=make_sn(dim,i);
    fac=lscale^(2*i)*coeff(i+1);
    wm=wm+fac*add;
  end
end
%----------------------------------------------------------------------------
