function [smp,smpvec,res2,soln2] = ridge_plot_box(a,b,we,wm_or_c,is_wm)

% @(#)   ridge_plot_box.m  version 1.5  created 92/12/15
%----------------------------------------------------------------------
% FUNCTION [smp,smpvec,res2,soln2] = ridge_plot_box(a,b,we,wm_or_c,is_wm)
%
% Produces ridge plots, that is, plots scaled residual e'*We*e and
% scaled solution x'*Wm*x against squared ridge parameter.
%
% Input:  a         matrix					(MxN)
%         b         right-hand side vector			(Mx1)
%         we        square of row (equation) weight matrix	(MxM)
%         wm_or_c   norm/semi-norm or column weight matrix	(NxN)
%         is_wm     1 if wm_or_c is norm/semi-norm weight matrix
%                   otherwise, assumed to be column weight matrix
%
% Output: smp       chosen value of ridge (smoothing) parameter
%         smpvec    vector of ridge parameters 
%         res2      vector of scaled residual size
%         soln2     vector of scaled solution size
%
% NOTE:   smp defined as in A'*A + smp^2*I
%
% Calls:  gcv_method_box, save_graph_box, seeplot_box
%----------------------------------------------------------------------

% Reference: Hoerl and Kennard (1970a,b)
% Author:    Peter McIntosh
% History:   See SCCS comments

%----------------------------------------------------------------------

%...Hardwired number of points in ridge plot...

nr=20;

%...Initialise plot arrays...

res2    = zeros(nr,1);
soln2   = zeros(nr,1);

%...Decide on method to use...

glb_gcv_method = gcv_method_box(we,wm_or_c,is_wm);

%...Do as much computation outside ridge loop as possible...

if glb_gcv_method==1
  if is_wm~=1,
    cinv = inv(wm_or_c);
    wm   = cinv*cinv;
  else
    wm   = wm_or_c;
  end
  at_we   = a'*we;
  glb_at_we_b = at_we*b;
  glb_at_we_a = at_we*a;
elseif glb_gcv_method==2
  if is_wm~=1,
    c   = wm_or_c;
    a_c = a*c;
  else
    wm        = wm_or_c;
    glb_wm_inv_at = wm\a';
  end
end

ridgeloop=1;
while ridgeloop==1
  lmin = input('RIDGE PLOT: enter lower limit for log10(smp) > ');
  lmax = input('RIDGE PLOT: enter upper limit for log10(smp) > ');

  smpvec  = logspace(lmin,lmax,nr)';
  smpvec2 = smpvec.*smpvec;
  
  for i=1:nr
    if glb_gcv_method==1
      bracket  = glb_at_we_a+smpvec2(i)*wm;
      x        = bracket\glb_at_we_b;
      soln2(i) = x'*wm*x;
    elseif glb_gcv_method==2
      if is_wm~=1,
	bracket  = a_c*a_c'+smpvec2(i)*inv(we);
	xp       = a_c'*(bracket\b);
	x        = c*xp;
        soln2(i) = xp'*xp;
      else
	bracket  = a*glb_wm_inv_at+smpvec2(i)*inv(we);
	x        = glb_wm_inv_at*(bracket\b);
        soln2(i) = x'*wm*x;
      end
    end
    e       = a*x-b;
    res2(i) = e'*we*e;
  end

  clg;

%...Graphs ordered so that ginput works off scale of last graph...
  
  subplot(222);
  loglog(soln2,res2);
  xlabel('soln^2');
  ylabel('res^2');
  title('e''*We*e vs x''*Wm*x');

  subplot(223);
  loglog(smpvec,soln2);
  xlabel('smp');
  ylabel('soln^2');
  title('Ridge plot: x''*Wm*x');
  
  subplot(221);
  loglog(smpvec,res2);
  xlabel('smp');
  ylabel('res^2');
  title('Ridge plot: e''*We*e');

  hold off;
  seeplot_box;

  reply=menu('RIDGE PLOT (CR to exit)',...
             'Proceed to choosing smp using mouse',...
             'Re-enter smp range');

  if reply==1
    break;
  elseif reply==2
    % Will continue in loop to re-enter smp range
  elseif isempty(reply)
    subplot(111);
    return;
  else
    % Will also continue in loop to re-enter smp range
  end

end

limits=axis;axis;

inputloop=1;
while inputloop==1
  checkloop=1;
  while checkloop==1
    disp(' ')
    disp('Use crosshairs to choose smoothing parameter from 1st graph only')
    smp=ginput(1);
%   ...Test that smp was chosen from 1st graph only...
    if smp <= 10^limits(2) & smp >= 10^limits(1)
      break;
    end
  end

  disp(' ')
  disp(['Smoothing parameter chosen = ',num2str(smp)])
  reply=input('Use this smoothing parameter ? (y/n) > ','s');
  if strcmp(reply,'y')
    break;
  end
end

save_graph_box;
subplot(111);
%----------------------------------------------------------------------
