function [x,r_model,r_data,s] = ridge_soln_box(a,b,we,wm_or_c,is_wm,smp)

% @(#)   ridge_soln_box.m  version 1.2  created 92/10/20
%-----------------------------------------------------------------------------
% FUNCTION [x,r_model,r_data,s] = ridge_soln_box(a,b,we,wm_or_c,is_wm,smp)
%
% Solves Ax=b using ridge analysis by minimizing e'*We*e + smp^2 * x'*Wm*x
% where e = A*x - b, Wm is the square of the model norm/semi-norm weight
% matrix (=C^-2 where C is the column weight matrix), and We is the square
% of the row (equation) weight matrix. Trade-off parameter smp is specified.
%
% Solution is  x = [A'*We*A + smp^2*Wm]^-1 * A'*We*b		(method 1)
%
%          or  x = Wm^-1*A'*[A*Wm^-1*A' + smp^2*We^-1]^-1 *b	(method 2)
%
% Input:  a         matrix					(MxN)
%         b         right-hand side vector			(Mx1)
%         we        square of row (equation) weight matrix	(MxM)
%         wm_or_c   norm/semi-norm or column weight matrix	(NxN)
%         is_wm     1 if wm_or_c is norm/semi-norm weight matrix
%                   else assumed to be column weight matrix	(scalar)
%         smp       smoothing (ridge) parameter			(scalar)
%
% Output: x         model solution				(Nx1)
%         r_model   model resolution matrix			(NxN)
%         r_data    data resolution matrix			(MxM)
%         s         degree of smoothing of data (%)		(scalar)
%
% Calls:  gcv_method_box
%-----------------------------------------------------------------------------

% Reference: Menke(1984) pp. 54-55, McIntosh & Veronis (1993)
% Author:    Peter McIntosh
% History:   See SCCS comments

%-----------------------------------------------------------------------------

%...Decide on method to use...

glb_gcv_method = gcv_method_box(we,wm_or_c,is_wm);

%...4 different ways of obtaining a solution...

%   Note: All methods assume weight matrices are symmetric
%         This is checked in gcv_method_box

[m,n] = size(a);
smp2  = smp*smp;

if glb_gcv_method==1

  if is_wm~=1,
    cinv = inv(wm_or_c);
    wm   = cinv*cinv;
  else
    wm   = wm_or_c;
  end

  at_we   = a'*we;
  glb_at_we_a = at_we*a;
  bracket = glb_at_we_a+smp2*wm;
  a_g     = bracket\at_we;

elseif glb_gcv_method==2

  if is_wm~=1,
    c       = wm_or_c;
    a_c     = a*c;
    bracket = a_c*a_c'+smp2*inv(we);
    a_g     = c*a_c'/bracket;
  else
    glb_wm_inv_at = wm_or_c\a';
    bracket   = a*glb_wm_inv_at+smp2*inv(we);
    a_g       = glb_wm_inv_at/bracket;
  end

end

x       = a_g*b;
r_model = a_g*a;
r_data  = a*a_g;
s       = 100*(1-trace(r_data)/m);

%-----------------------------------------------------------------------------
