function [qypath,qxpath,qt,qy,qx] = ekman(taux,tauy, lat, long )

%============================================================================
% EKMAN   3.3   92/04/23  Copyright (C) Phil Morgan
%
% [qypath, qxpath, qt, qy, qx] = ekman(taux, tauy, lat, long )
%
% DESCRIPTION:
%   Calculates the ekman flux at the midpoints between positions
%   along a voyage track.  Uses eqn 9.16 of Ponds & Pickard 1986.
%
% INPUT:
%   taux   = x component wind stress (dynes/cm^2)             DIM(n,1)
%   tauy   = y component wind stress                          DIM(n,1)
%   lat    = latitudes  for each tauy (decimal degrees +ve N) DIM(n,1)
%   long   = longitudes for each taux (decimal degrees +ve E) DIM(n,1)
% 
%   Note:  1 dyne/cm^2 = 0.1 N/m^2
%
% OUTPUT:
%   qypath = ekman tranports at the midpoints in the 'y' direction
%   qxpath = ekman tranports at the midpoints in the 'x' direction
%   qt	   = total ekman transport along path [m^3/s]
%   qy     = y component ekman transport (per metre) in x,y (N,E) co-ords
%   qx     = x component ekman transport (per metre) in x,y (N,E) co-ords
%            Units [m^2/s]
%
% REFERENCE:
%   S. Ponds & G.L. Pickard 1986
%   "Introduction to Dynamical Oceanography"  (2nd Edition)
%   Pergamon Press, Oxford(UK)  ISBN: 0-08-028728-X
%
% CALLER:  general purpose
% CALLEE:  distance.m phase2cart.m bear2cart.m 
%
% AUTHOR:   Phil Morgan 25-02-92
%==================================================================

 
% SET CONSTANTS
jay     = sqrt(-1);
DEG2RAD = pi/180;
RAD2DEG = 180/pi;

CGS2SI  = 0.1;     
RHO     = 1000;    
OMEGA   = 7.29e-5; 

% FORCE INPUT VARIABLES TO COLUMN VECTOR FOR DEBUGGING PURPOSES
[m,n]=size(lat);
if m==1
  lat   = lat';
  long  = long';
end %if

[m,n]=size(taux);
if m==1
  taux   = taux';
  tauy   = tauy';
end %if


% BEGIN
n  = length(lat);
f  = 2*OMEGA*sin( (lat(1:n-1)+lat(2:n))*DEG2RAD/2 );

% CALCULATE RESULTANT EKMAN FLUX VECTOR FROM COMPONENTS
qx    = +(CGS2SI/RHO)*((tauy(1:n-1)+tauy(2:n))/2) ./ f;
qy    = -(CGS2SI/RHO)*((taux(1:n-1)+taux(2:n))/2) ./ f;
qqz   = qx+qy*jay;
alpha = phase2cart( angle(qqz)*RAD2DEG );
r     = abs(qqz);

% CLACULATE CRUISE TRACK VECTORS
[distkm, bearing] = distance(lat,long,'km');
distm             = 1000*distkm;
beta              = bear2cart(bearing);
pathz             = r .* exp( jay*DEG2RAD*beta); % only used r so scale ok.
                                                 % direction is important
% CALCULATE NEW X,Y CO-ORDS AFTER ROTATING AXIS BY "-beta" DEGREES
newqx = r .* cos(  (alpha - beta)*DEG2RAD );
newqy = r .* sin(  (alpha - beta)*DEG2RAD );
newqz = newqx+newqy*jay;

% EKMAN FLUX NORMAL TO CRUISE TRACK VECTOR
qypath = distm .* newqy;
qxpath = distm .* newqx;
good  = find(~isnan(qypath));
qt    = sum(qypath(good));

%--------------------------------------------------------------------
